@@ -8,5 +8,8 @@ autoconf
 
 echo "
 suggested configure parameters:
+# prepare for rpmbuild, only generate spec files
+./configure --with-km --enable-spec
+# or prepare for direct build
 ./configure --prefix=/usr --localstatedir=/var --sysconfdir=/etc
 "
