@@ -1,6 +1,6 @@
 #include <linux/types.h>
 
-void foo()
+void foo(void)
 {
 	fmode_t mode;
 }
