@@ -1,6 +1,6 @@
 #include <linux/cpumask.h>
 
-void foo()
+void foo(void)
 {
 	int x = nr_cpu_ids;
 }
