@@ -1,10 +1,20 @@
 #ifndef _DRBD_WRAPPERS_H
 #define _DRBD_WRAPPERS_H
 
+#include "compat.h"
 #include <linux/ctype.h>
 #include <linux/net.h>
-
 #include <linux/version.h>
+#include <linux/crypto.h>
+#include <linux/netlink.h>
+#include <linux/idr.h>
+#include <linux/fs.h>
+#include <linux/bio.h>
+#include <linux/slab.h>
+#include <linux/completion.h>
+#include <linux/proc_fs.h>
+#include <linux/blkdev.h>
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
 # error "At least kernel version 2.6.18 (with patches) required"
 #endif
@@ -17,7 +27,6 @@
    It had 4 arguments before dd3932eddf428571762596e17b65f5dc92ca361b,
    after it got 3 arguments. (With that commit came BLKDEV_DISCARD_SECURE
    and BLKDEV_IFL_WAIT disappeared again.) */
-#include <linux/blkdev.h>
 #ifndef BLKDEV_IFL_WAIT
 #ifndef BLKDEV_DISCARD_SECURE
 /* before fbd9b09a177 */
@@ -29,14 +38,6 @@
 #define blkdev_issue_flush(b, gfpf, s)	blkdev_issue_flush(b, gfpf, s, BLKDEV_IFL_WAIT)
 #endif
 
-#include <linux/fs.h>
-#include <linux/bio.h>
-#include <linux/slab.h>
-#include <linux/completion.h>
-
-/* for the proc_create wrapper */
-#include <linux/proc_fs.h>
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,31)
 static inline unsigned short queue_logical_block_size(struct request_queue *q)
 {
@@ -67,13 +68,6 @@ static inline void blk_queue_logical_block_size(struct request_queue *q, unsigne
 }
 #endif
 
-/* Returns the number of 512 byte sectors of the device */
-static inline sector_t drbd_get_capacity(struct block_device *bdev)
-{
-	/* return bdev ? get_capacity(bdev->bd_disk) : 0; */
-	return bdev ? i_size_read(bdev->bd_inode) >> 9 : 0;
-}
-
 #ifdef COMPAT_HAVE_VOID_MAKE_REQUEST
 /* in Commit 5a7bbad27a410350e64a2d7f5ec18fc73836c14f (between Linux-3.1 and 3.2)
    make_request() becomes type void. Before it had type int. */
@@ -84,17 +78,6 @@ static inline sector_t drbd_get_capacity(struct block_device *bdev)
 #define MAKE_REQUEST_RETURN return 0
 #endif
 
-#include "drbd_int.h"
-
-/* sets the number of 512 byte sectors of our virtual device */
-static inline void drbd_set_my_capacity(struct drbd_conf *mdev,
-					sector_t size)
-{
-	/* set_capacity(mdev->this_bdev->bd_disk, size); */
-	set_capacity(mdev->vdisk, size);
-	mdev->this_bdev->bd_inode->i_size = (loff_t)size << 9;
-}
-
 #ifndef COMPAT_HAVE_FMODE_T
 typedef unsigned __bitwise__ fmode_t;
 #endif
@@ -215,66 +198,9 @@ static inline void sg_set_page(struct scatterlist *sg, struct page *page,
 # endif
 # define disk_to_kobj(disk) (&disk_to_dev(disk)->kobj)
 #endif
-static inline void drbd_kobject_uevent(struct drbd_conf *mdev)
+
+static inline int drbd_backing_bdev_events(struct gendisk *disk)
 {
-	kobject_uevent(disk_to_kobj(mdev->vdisk), KOBJ_CHANGE);
-	/* rhel4 / sles9 and older don't have this at all,
-	 * which means user space (udev) won't get events about possible changes of
-	 * corresponding resource + disk names after the initial drbd minor creation.
-	 */
-}
-
-
-/*
- * used to submit our private bio
- */
-static inline void drbd_generic_make_request(struct drbd_conf *mdev,
-					     int fault_type, struct bio *bio)
-{
-	__release(local);
-	if (!bio->bi_bdev) {
-		printk(KERN_ERR "drbd%d: drbd_generic_make_request: "
-				"bio->bi_bdev == NULL\n",
-		       mdev_to_minor(mdev));
-		dump_stack();
-		bio_endio(bio, -ENODEV);
-		return;
-	}
-
-	if (drbd_insert_fault(mdev, fault_type))
-		bio_endio(bio, -EIO);
-	else
-		generic_make_request(bio);
-}
-
-/* see 7eaceac block: remove per-queue plugging */
-#ifdef blk_queue_plugged
-static inline void drbd_plug_device(struct drbd_conf *mdev)
-{
-	struct request_queue *q;
-	q = bdev_get_queue(mdev->this_bdev);
-
-	spin_lock_irq(q->queue_lock);
-
-/* XXX the check on !blk_queue_plugged is redundant,
- * implicitly checked in blk_plug_device */
-
-	if (!blk_queue_plugged(q)) {
-		blk_plug_device(q);
-		del_timer(&q->unplug_timer);
-		/* unplugging should not happen automatically... */
-	}
-	spin_unlock_irq(q->queue_lock);
-}
-#else
-static inline void drbd_plug_device(struct drbd_conf *mdev)
-{
-}
-#endif
-
-static inline int drbd_backing_bdev_events(struct drbd_conf *mdev)
-{
-	struct gendisk *disk = mdev->ldev->backing_bdev->bd_contains->bd_disk;
 #if defined(__disk_stat_inc)
 	/* older kernel */
 	return (int)disk_stat_read(disk, sectors[0])
@@ -519,8 +445,8 @@ static inline int zalloc_cpumask_var(cpumask_var_t *mask, gfp_t flags)
 
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
-#define __bitmap_parse(BUF, BUFLEN, ISUSR, MASKP, NMASK) \
-	backport_bitmap_parse(BUF, BUFLEN, ISUSR, MASKP, NMASK)
+#define bitmap_parse(BUF, BUFLEN, MASKP, NMASK) \
+	backport_bitmap_parse(BUF, BUFLEN, 0, MASKP, NMASK)
 
 #define CHUNKSZ                         32
 #define nbits_to_hold_value(val)        fls(val)
@@ -1229,4 +1155,130 @@ static inline void list_splice_tail_init(struct list_head *list,
 }
 #endif
 
+#ifndef COMPAT_HAVE_IDR_ALLOC
+static inline int idr_alloc(struct idr *idr, void *ptr, int start, int end, gfp_t gfp_mask)
+{
+	int rv, got;
+
+	if (!idr_pre_get(idr, gfp_mask))
+		return -ENOMEM;
+	rv = idr_get_new_above(idr, ptr, start, &got);
+	if (rv < 0)
+		return rv;
+
+	if (got >= end) {
+		idr_remove(idr, got);
+		return -ENOSPC;
+	}
+
+	return got;
+}
+#endif
+
+#ifndef COMPAT_HAVE_IDR_FOR_EACH_ENTRY
+/**
+ * idr_for_each_entry - iterate over an idr's elements of a given type
+ * @idp:     idr handle
+ * @entry:   the type * to use as cursor
+ * @id:      id entry's key
+ *
+ * @entry and @id do not need to be initialized before the loop, and
+ * after normal terminatinon @entry is left with the value NULL.  This
+ * is convenient for a "not found" value.
+  */
+#define idr_for_each_entry(idp, entry, id)                     \
+       for (id = 0; ((entry) = idr_get_next(idp, &(id))) != NULL; ++id)
+#endif
+
+#ifndef COMPAT_HAVE_PRANDOM_U32
+static inline u32 prandom_u32(void)
+{
+	return random32();
+}
+#endif
+
+#ifndef COMPAT_HAVE_PROC_PDE_DATA
+#define PDE_DATA(inode) PDE(inode)->data
+#endif
+
+#ifndef COMPAT_HAVE_TASK_PID_NR
+#include <linux/sched.h>
+
+static inline pid_t task_pid_nr(struct task_struct *tsk)
+{
+	return tsk->pid;
+}
+#endif
+
+#ifndef for_each_cpu
+# define for_each_cpu(cpu, mask) for_each_cpu_mask(cpu, mask)
+#endif
+
+#ifndef COMPAT_HAVE_CPUMASK_EMPTY
+#include <linux/cpumask.h>
+#define cpumask_empty(mask) cpus_empty(mask)
+#endif
+
+
+#if !defined(QUEUE_FLAG_DISCARD) || !defined(QUEUE_FLAG_SECDISCARD)
+# define queue_flag_set_unlocked(F, Q)				\
+	({							\
+		if ((F) != -1)					\
+			queue_flag_set_unlocked(F, Q);		\
+	})
+
+# define queue_flag_clear_unlocked(F, Q)			\
+	({							\
+		if ((F) != -1)					\
+			queue_flag_clear_unlocked(F, Q);	\
+	})
+
+# ifndef blk_queue_discard
+#  define blk_queue_discard(q)   (0)
+#  define QUEUE_FLAG_DISCARD    (-1)
+# endif
+
+# ifndef blk_queue_secdiscard
+#  define blk_queue_secdiscard(q)   (0)
+#  define QUEUE_FLAG_SECDISCARD    (-1)
+# endif
+#endif
+
+
+#ifndef BLKDEV_ISSUE_ZEROOUT_EXPORTED
+/* Was introduced with 2.6.34 */
+extern int blkdev_issue_zeroout(struct block_device *bdev, sector_t sector,
+				sector_t nr_sects, gfp_t gfp_mask);
+#else
+#ifdef COMPAT_BLKDEV_ISSUE_ZEROOUT_HAS_5_PARAMTERS
+/* ... but in 2.6.34 and 2.6.35 it had 5 parameters. Later only 4 */
+#define blkdev_issue_zeroout(BDEV, SS, NS, GFP) \
+	blkdev_issue_zeroout(BDEV, SS, NS, GFP, BLKDEV_IFL_WAIT)
+
+#endif
+#endif
+
+
+#ifndef COMPAT_HAVE_GENL_LOCK
+static inline void genl_lock(void)  { }
+static inline void genl_unlock(void)  { }
+#endif
+
+#ifdef COMPAT_HAVE_STRUCT_QUEUE_LIMITS
+#define DRBD_QUEUE_LIMITS(q) (&(q)->limits)
+#define LIMIT_TYPE struct queue_limits
+#else
+#define DRBD_QUEUE_LIMITS(q) (q)
+#define LIMIT_TYPE struct request_queue
+#endif
+
+#ifndef COMPAT_HAVE_BLK_SET_STACKING_LIMITS
+static inline void blk_set_stacking_limits(LIMIT_TYPE *lim)
+{
+# ifdef COMPAT_QUEUE_LIMITS_HAS_DISCARD_ZEROES_DATA
+	lim->discard_zeroes_data = 1;
+# endif
+}
+#endif
+
 #endif
