@@ -6,7 +6,7 @@
  * and we want to avoid to introduce incompatibilities by including kernel
  * headers from user space.
  *
- * we need the u32 and u64 types,
+ * we need the uint32_t and uint64_t types,
  * the hamming weight functions,
  * and the cpu_to_le etc. endianness convert functions.
  */
@@ -18,19 +18,10 @@
 # define BITS_PER_LONG __WORDSIZE
 #endif
 
-#define u64 uint64_t
-#define s64 int64_t
-#define u32 uint32_t
-#define s32 int32_t
-#define u16 uint16_t
-#define __u64 uint64_t
-#define __u32 uint32_t
-#define __u16 uint16_t
-
 /* linux/byteorder/swab.h */
 
 /* casts are necessary for constants, because we never know for sure
- * how U/UL/ULL map to __u16, __u32, __u64. At least not in a portable way.
+ * how U/UL/ULL map to __u16, uint32_t, uint64_t. At least not in a portable way.
  */
 
 /*
@@ -48,26 +39,26 @@
 
 #define __swab32(x) \
 ({ \
-	__u32 __x = (x); \
-	((__u32)( \
-		(((__u32)(__x) & (__u32)0x000000ffUL) << 24) | \
-		(((__u32)(__x) & (__u32)0x0000ff00UL) <<  8) | \
-		(((__u32)(__x) & (__u32)0x00ff0000UL) >>  8) | \
-		(((__u32)(__x) & (__u32)0xff000000UL) >> 24) )); \
+	uint32_t __x = (x); \
+	((uint32_t)( \
+		(((uint32_t)(__x) & (uint32_t)0x000000ffUL) << 24) | \
+		(((uint32_t)(__x) & (uint32_t)0x0000ff00UL) <<  8) | \
+		(((uint32_t)(__x) & (uint32_t)0x00ff0000UL) >>  8) | \
+		(((uint32_t)(__x) & (uint32_t)0xff000000UL) >> 24) )); \
 })
 
 #define __swab64(x) \
 ({ \
-	__u64 __x = (x); \
-	((__u64)( \
-		(__u64)(((__u64)(__x) & (__u64)0x00000000000000ffULL) << 56) | \
-		(__u64)(((__u64)(__x) & (__u64)0x000000000000ff00ULL) << 40) | \
-		(__u64)(((__u64)(__x) & (__u64)0x0000000000ff0000ULL) << 24) | \
-		(__u64)(((__u64)(__x) & (__u64)0x00000000ff000000ULL) <<  8) | \
-		(__u64)(((__u64)(__x) & (__u64)0x000000ff00000000ULL) >>  8) | \
-		(__u64)(((__u64)(__x) & (__u64)0x0000ff0000000000ULL) >> 24) | \
-		(__u64)(((__u64)(__x) & (__u64)0x00ff000000000000ULL) >> 40) | \
-		(__u64)(((__u64)(__x) & (__u64)0xff00000000000000ULL) >> 56) )); \
+	uint64_t __x = (x); \
+	((uint64_t)( \
+		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00000000000000ffULL) << 56) | \
+		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x000000000000ff00ULL) << 40) | \
+		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x0000000000ff0000ULL) << 24) | \
+		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00000000ff000000ULL) <<  8) | \
+		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x000000ff00000000ULL) >>  8) | \
+		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x0000ff0000000000ULL) >> 24) | \
+		(uint64_t)(((uint64_t)(__x) & (uint64_t)0x00ff000000000000ULL) >> 40) | \
+		(uint64_t)(((uint64_t)(__x) & (uint64_t)0xff00000000000000ULL) >> 56) )); \
 })
 
 /*
@@ -76,10 +67,10 @@
  */
 
 #if __BYTE_ORDER == __LITTLE_ENDIAN
-#define cpu_to_le64(x) ((__u64)(x))
-#define le64_to_cpu(x) ((__u64)(x))
-#define cpu_to_le32(x) ((__u32)(x))
-#define le32_to_cpu(x) ((__u32)(x))
+#define cpu_to_le64(x) ((uint64_t)(x))
+#define le64_to_cpu(x) ((uint64_t)(x))
+#define cpu_to_le32(x) ((uint32_t)(x))
+#define le32_to_cpu(x) ((uint32_t)(x))
 #define cpu_to_le16(x) ((__u16)(x))
 #define le16_to_cpu(x) ((__u16)(x))
 #define cpu_to_be64(x) __swab64((x))
@@ -95,10 +86,10 @@
 # define le32_to_cpu(x) __swab32((x))
 # define cpu_to_le16(x) __swab16((x))
 # define le16_to_cpu(x) __swab16((x))
-# define cpu_to_be64(x) ((__u64)(x))
-# define be64_to_cpu(x) ((__u64)(x))
-# define cpu_to_be32(x) ((__u32)(x))
-# define be32_to_cpu(x) ((__u32)(x))
+# define cpu_to_be64(x) ((uint64_t)(x))
+# define be64_to_cpu(x) ((uint64_t)(x))
+# define cpu_to_be32(x) ((uint32_t)(x))
+# define be32_to_cpu(x) ((uint32_t)(x))
 # define cpu_to_be16(x) ((__u16)(x))
 # define be16_to_cpu(x) ((__u16)(x))
 #else
@@ -133,13 +124,13 @@ static inline unsigned int generic_hweight32(unsigned int w)
 	return (res & 0x0000FFFF) + ((res >> 16) & 0x0000FFFF);
 }
 
-static inline unsigned long generic_hweight64(__u64 w)
+static inline unsigned long generic_hweight64(uint64_t w)
 {
 #if BITS_PER_LONG < 64
 	return generic_hweight32((unsigned int)(w >> 32)) +
 				generic_hweight32((unsigned int)w);
 #else
-	u64 res;
+	uint64_t res;
 	res = (w & 0x5555555555555555) + ((w >> 1) & 0x5555555555555555);
 	res = (res & 0x3333333333333333) + ((res >> 2) & 0x3333333333333333);
 	res = (res & 0x0F0F0F0F0F0F0F0F) + ((res >> 4) & 0x0F0F0F0F0F0F0F0F);
