@@ -46,6 +46,10 @@ enum usage_count_type {
   UC_ASK,
 };
 
+enum pp_flags {
+	MATCH_ON_PROXY = 1,
+};
+
 struct d_globals
 {
   int disable_io_hints;
@@ -83,6 +87,8 @@ struct d_proxy_info
   char* outside_addr;
   char* outside_port;
   char* outside_af;
+  struct d_option *options; /* named proxy_options in other places */
+  struct d_option *plugins; /* named proxy_plugins in other places */
 };
 
 struct d_volume
@@ -208,7 +214,7 @@ static inline int m_system_ex(char **argv, int flags, const char *res_name)
 	return ex;
 }
 extern struct d_option* find_opt(struct d_option*,char*);
-extern void validate_resource(struct d_resource *);
+extern void validate_resource(struct d_resource *, enum pp_flags);
 /* stages of configuration, as performed on "drbdadm up"
  * or "drbdadm adjust":
  */
@@ -255,9 +261,6 @@ enum pr_flags {
   NoneHAllowed  = 4,
   PARSE_FOR_ADJUST = 8
 };
-enum pp_flags {
-	match_on_proxy = 1,
-};
 
 extern struct d_resource* parse_resource_for_adjust(struct cfg_ctx *ctx);
 extern struct d_resource* parse_resource(char*, enum pr_flags);
@@ -274,8 +277,11 @@ extern void set_me_in_resource(struct d_resource* res, int match_on_proxy);
 extern void set_peer_in_resource(struct d_resource* res, int peer_required);
 extern void set_on_hosts_in_res(struct d_resource *res);
 extern void set_disk_in_res(struct d_resource *res);
-extern char *proxy_connection_name(struct d_resource *res);
-int parse_proxy_settings(struct d_resource *res, int check_proxy_token);
+extern int _proxy_connect_name_len(struct d_resource *res);
+extern char *_proxy_connection_name(char *conn_name, struct d_resource *res);
+#define proxy_connection_name(RES) \
+	_proxy_connection_name(alloca(_proxy_connect_name_len(RES)), RES)
+int parse_proxy_options_section(struct d_resource *res);
 /* conn_name is optional and mostly for compatibility with dcmd */
 int do_proxy_conn_up(struct cfg_ctx *ctx);
 int do_proxy_conn_down(struct cfg_ctx *ctx);
@@ -373,8 +379,5 @@ extern void add_setup_option(bool explicit, char *option);
 })
 
 
-#define PARSER_CHECK_PROXY_KEYWORD (1)
-#define PARSER_STOP_IF_INVALID (2)
-
 #endif
 
