@@ -157,6 +157,7 @@ static int opts_equal(struct context_def *ctx, struct d_option* conf, struct d_o
 static int addr_equal(struct d_resource* conf, struct d_resource* running)
 {
 	int equal;
+	char *peer_addr, *peer_af, *peer_port;
 
 	if (conf->peer == NULL && running->peer == NULL) return 1;
 	if (running->peer == NULL) return 0;
@@ -165,16 +166,29 @@ static int addr_equal(struct d_resource* conf, struct d_resource* running)
 		!strcmp(conf->me->port,           running->me->port) &&
 		!strcmp(conf->me->address_family, running->me->address_family);
 
-	if(conf->me->proxy)
-		equal = equal &&
-			!strcmp(conf->me->proxy->inside_addr, running->peer->address) &&
-			!strcmp(conf->me->proxy->inside_port, running->peer->port) &&
-			!strcmp(conf->me->proxy->inside_af,   running->peer->address_family);
-	else
-		equal = equal && conf->peer &&
-			!strcmp(conf->peer->address,        running->peer->address) &&
-			!strcmp(conf->peer->port,           running->peer->port) &&
-			!strcmp(conf->peer->address_family, running->peer->address_family);
+	if(conf->me->proxy) {
+		peer_addr = conf->me->proxy->inside_addr;
+		peer_port = conf->me->proxy->inside_port;
+		peer_af = conf->me->proxy->inside_af;
+	} else {
+		peer_addr = conf->peer->address;
+		peer_port = conf->peer->port;
+		peer_af = conf->peer->address_family;
+	}
+
+	equal = equal && conf->peer &&
+		!strcmp(peer_addr, running->peer->address) &&
+		!strcmp(peer_port, running->peer->port) &&
+		!strcmp(peer_af, running->peer->address_family);
+
+	if (verbose > 2)
+		fprintf(stderr, "Network addresses differ:\n"
+			"\trunning: %s:%s:%s -- %s:%s:%s\n"
+			"\t config: %s:%s:%s -- %s:%s:%s\n",
+			running->me->address_family, running->me->address, running->me->port,
+			running->peer->address_family, running->peer->address, running->peer->port,
+			conf->me->address_family, conf->me->address, conf->me->port,
+			peer_af, peer_addr, peer_port);
 
 	return equal;
 }
@@ -297,7 +311,7 @@ static int proxy_reconf(struct cfg_ctx *ctx, struct d_resource *running)
 		goto redo_whole_conn;
 
 	find_option_in_resources("memlimit",
-			res->proxy_options, &res_o,
+			res->me->proxy->options, &res_o,
 			running->proxy_options, &run_o,
 			NULL, NULL);
 	v1 = res_o ? m_strtoll(res_o->value, 1) : 0;
@@ -322,7 +336,7 @@ redo_whole_conn:
 	}
 
 
-	res_o = res->proxy_plugins;
+	res_o = res->me->proxy->plugins;
 	run_o = running->proxy_plugins;
 	used = 0;
 	conn_name = proxy_connection_name(res);
@@ -667,8 +681,7 @@ int adm_adjust(struct cfg_ctx *ctx)
 
 		/* actually parse "drbd-proxy-ctl show" output */
 		yyin = m_popen(&pid,argv);
-		can_do_proxy = !parse_proxy_settings(running,
-				PARSER_CHECK_PROXY_KEYWORD | PARSER_STOP_IF_INVALID);
+		can_do_proxy = !parse_proxy_options_section(running);
 		fclose(yyin);
 
 		waitpid(pid,0,0);
@@ -690,8 +703,7 @@ int adm_adjust(struct cfg_ctx *ctx)
 	if (ctx->res->me->proxy && can_do_proxy)
 		do_connect |= proxy_reconf(ctx, running);
 
-	if (do_connect && running)
-		do_disconnect = running->net_options != NULL;
+	do_disconnect = do_connect && running && (running->peer || running->net_options);
 
 	if (do_res_options)
 		schedule_deferred_cmd(adm_set_default_res_options, ctx, "resource-options", CFG_RESOURCE);
@@ -716,8 +728,12 @@ int adm_adjust(struct cfg_ctx *ctx)
 	}
 
 	if (do_connect) {
-		if (do_disconnect && ctx->res->peer)
-			schedule_deferred_cmd(adm_disconnect, ctx, "disconnect", CFG_NET_PREREQ);
+		/* "disconnect" specifying the end-point addresses currently in-use,
+		 * before "connect"ing with the addresses currently in-config-file. */
+		if (do_disconnect) {
+			struct cfg_ctx tmp_ctx = { .res = running, .vol = vol, };
+			schedule_deferred_cmd(adm_disconnect, &tmp_ctx, "disconnect", CFG_NET_PREREQ);
+		}
 		schedule_deferred_cmd(adm_connect, ctx, "connect", CFG_NET);
 		do_net_options = 0;
 	}
