@@ -47,6 +47,7 @@ YYSTYPE yylval;
 /////////////////////
 
 static int c_section_start;
+static int parse_proxy_options(struct d_option **, struct d_option **);
 void my_parse(void);
 
 struct d_name *names_from_str(char* str)
@@ -867,6 +868,9 @@ static void parse_proxy_section(struct d_host_info *host)
 		case TK_OUTSIDE:
 			parse_address(proxy->on_hosts, &proxy->outside_addr, &proxy->outside_port, &proxy->outside_af);
 			break;
+		case TK_OPTIONS:
+			parse_proxy_options(&proxy->options, &proxy->plugins);
+			break;
 		case '}':
 			goto break_loop;
 		default:
@@ -1754,7 +1758,7 @@ void set_disk_in_res(struct d_resource *res)
 
 void proxy_delegate(void *ctx)
 {
-	struct d_resource *res = (struct d_resource *)ctx;
+	struct d_option **proxy_plugins = (struct d_option **)ctx;
 	int token;
 	struct d_option *options, *opt;
 	struct d_name *line, *word, **pnp;
@@ -1771,6 +1775,7 @@ void proxy_delegate(void *ctx)
 	while (1) {
 		pnp = &line;
 		while (1) {
+			yylval.txt = NULL;
 			token = yylex();
 			if (token == ';')
 				break;
@@ -1786,7 +1791,7 @@ void proxy_delegate(void *ctx)
 			word = malloc(sizeof(struct d_name));
 			if (!word)
 				pdperror("out of memory.");
-			word->name = yylval.txt;
+			word->name = yylval.txt ? yylval.txt : strdup(yytext);
 			word->next = NULL;
 			*pnp = word;
 			pnp = &word->next;
@@ -1800,41 +1805,37 @@ void proxy_delegate(void *ctx)
 		free_names(line);
 	}
 out:
-	if (res)
-		res->proxy_plugins = options;
+	if (proxy_plugins)
+		*proxy_plugins = options;
 }
 
-int parse_proxy_settings(struct d_resource *res, int flags)
+static int parse_proxy_options(struct d_option **proxy_options, struct d_option **proxy_plugins)
 {
-	int token;
-	struct d_option *proxy_options;
-
-	if (flags & PARSER_CHECK_PROXY_KEYWORD) {
-		token = yylex();
-		if (token != TK_PROXY) {
-			if (flags & PARSER_STOP_IF_INVALID) {
-				yyrestart(yyin); /* flushes flex's buffers */
-				return 1;
-			}
-
-			pe_expected_got("proxy", token);
-		}
-	}
+	struct d_option *opts;
 
 	EXP('{');
+	opts = parse_options_d(0, 0, TK_PROXY_OPTION | TK_PROXY_GROUP,
+			       TK_PROXY_DELEGATE, proxy_delegate, proxy_plugins);
 
-	proxy_options = parse_options_d(0,
-					0,
-					TK_PROXY_OPTION | TK_PROXY_GROUP,
-					TK_PROXY_DELEGATE,
-					proxy_delegate,
-					res);
+	if (proxy_options)
+		*proxy_options = opts;
 
-	if (res)
-		res->proxy_options = proxy_options;
 	return 0;
 }
 
+int parse_proxy_options_section(struct d_resource *res)
+{
+	int token;
+
+	token = yylex();
+	if (token != TK_PROXY) {
+		yyrestart(yyin); /* flushes flex's buffers */
+		return 1;
+	}
+
+	return parse_proxy_options(&res->proxy_options, &res->proxy_plugins);
+}
+
 struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 {
 	struct d_resource* res;
@@ -1952,7 +1953,7 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 			break;
 		case TK_PROXY:
 			check_upr("proxy section", "%s:proxy", res->name);
-			parse_proxy_settings(res, 0);
+			parse_proxy_options(&res->proxy_options, &res->proxy_plugins);
 			break;
 		case TK_DEVICE:
 			check_upr("device statement", "%s:device", res->name);
@@ -2035,12 +2036,12 @@ void post_parse(struct d_resource *config, enum pp_flags flags)
 	/* Needs "on_hosts" and host->lower already set */
 	for_each_resource(res, tmp, config)
 		if (!res->stacked_on_one)
-			set_me_in_resource(res, flags & match_on_proxy);
+			set_me_in_resource(res, flags & MATCH_ON_PROXY);
 
 	/* Needs host->lower->me already set */
 	for_each_resource(res, tmp, config)
 		if (res->stacked_on_one)
-			set_me_in_resource(res, flags & match_on_proxy);
+			set_me_in_resource(res, flags & MATCH_ON_PROXY);
 
 	// Needs "me" set already
 	for_each_resource(res, tmp, config)
