@@ -27,6 +27,8 @@ enum yytokentype {
 	TK_INVALID,
 	TK_INVALID_CHAR,
 	TK_LA_BIO_SIZE,
+	TK_AL_STRIPES,
+	TK_AL_STRIPE_SIZE_4K,
 };
 
 /* avoid compiler warnings about implicit declaration */
