@@ -400,6 +400,7 @@ struct drbd_cmd commands[] = {
 	{"verify", P_start_ov, F_CONFIG_CMD, {{ NULL,
 	 (struct drbd_option[]) {
 		 { "start",'s',T_start_sector, EN(DISK_SIZE_SECT,'s',"bytes") },
+		 { "stop",'S',T_stop_sector, EN(DISK_SIZE_SECT,'s',"bytes") },
 		 CLOSE_OPTIONS }} }, },
 	{"down",            0, down_cmd, get_usage, { {NULL, NULL }} },
 	{"state", P_get_state, F_GET_CMD, { .gp={ role_scmd} } },
@@ -440,8 +441,8 @@ static const char *error_messages[] = {
 	EM(ERR_MD_NOT_BDEV) = "Meta device is not a block device.",
 	EM(109) = "Open of lower device failed.",
 	EM(110) = "Open of meta device failed.",
-	EM(ERR_DISK_TO_SMALL) = "Low.dev. smaller than requested DRBD-dev. size.",
-	EM(ERR_MD_DISK_TO_SMALL) = "Meta device too small.",
+	EM(ERR_DISK_TOO_SMALL) = "Low.dev. smaller than requested DRBD-dev. size.",
+	EM(ERR_MD_DISK_TOO_SMALL) = "Meta device too small.",
 	EM(113) = "You have to use the disk command first.",
 	EM(ERR_BDCLAIM_DISK) = "Lower device is already claimed. This usually means it is mounted.",
 	EM(ERR_BDCLAIM_MD_DISK) = "Meta device is already claimed. This usually means it is mounted.",
@@ -456,7 +457,7 @@ static const char *error_messages[] = {
 	"the kernel. (Maybe you need to modprobe it, or modprobe hmac?)",
 	EM(ERR_AUTH_ALG_ND) = "The 'cram-hmac-alg' you specified is not a digest.",
 	EM(ERR_NOMEM) = "kmalloc() failed. Out of memory?",
-	EM(ERR_DISCARD) = "--discard-my-data not gllowed when primary.",
+	EM(ERR_DISCARD_IMPOSSIBLE) = "--discard-my-data not allowed when primary.",
 	EM(ERR_DISK_CONFIGURED) = "Device is attached to a disk (use detach first)",
 	EM(ERR_NET_CONFIGURED) = "Device has a net-config (use disconnect first)",
 	EM(ERR_MANDATORY_TAG) = "UnknownMandatoryTag",
@@ -1447,28 +1448,30 @@ static char *af_to_str(int af)
 
 static void show_address(void* address, int addr_len)
 {
-	struct sockaddr     *addr;
-	struct sockaddr_in  *addr4;
-	struct sockaddr_in6 *addr6;
+	union {
+		struct sockaddr     addr;
+		struct sockaddr_in  addr4;
+		struct sockaddr_in6 addr6;
+	} a;
 	char buffer[INET6_ADDRSTRLEN];
 
-	addr = (struct sockaddr *)address;
-	if (addr->sa_family == AF_INET
-	|| addr->sa_family == get_af_ssocks(0)
-	|| addr->sa_family == AF_INET_SDP) {
-		addr4 = (struct sockaddr_in *)address;
+	/* avoid alignment issues on certain platforms (e.g. armel) */
+	memset(&a, 0, sizeof(a));
+	memcpy(&a.addr, address, addr_len);
+	if (a.addr.sa_family == AF_INET
+	|| a.addr.sa_family == get_af_ssocks(0)
+	|| a.addr.sa_family == AF_INET_SDP) {
 		printf("\taddress\t\t\t%s %s:%d;\n",
-		       af_to_str(addr4->sin_family),
-		       inet_ntoa(addr4->sin_addr),
-		       ntohs(addr4->sin_port));
-	} else if (addr->sa_family == AF_INET6) {
-		addr6 = (struct sockaddr_in6 *)address;
+		       af_to_str(a.addr4.sin_family),
+		       inet_ntoa(a.addr4.sin_addr),
+		       ntohs(a.addr4.sin_port));
+	} else if (a.addr.sa_family == AF_INET6) {
 		printf("\taddress\t\t\t%s [%s]:%d;\n",
-		       af_to_str(addr6->sin6_family),
-		       inet_ntop(addr6->sin6_family, &addr6->sin6_addr, buffer, INET6_ADDRSTRLEN),
-		       ntohs(addr6->sin6_port));
+		       af_to_str(a.addr6.sin6_family),
+		       inet_ntop(a.addr6.sin6_family, &a.addr6.sin6_addr, buffer, INET6_ADDRSTRLEN),
+		       ntohs(a.addr6.sin6_port));
 	} else {
-		printf("\taddress\t\t\t[unknown af=%d, len=%d]\n", addr->sa_family, addr_len);
+		printf("\taddress\t\t\t[unknown af=%d, len=%d]\n", a.addr.sa_family, addr_len);
 	}
 }
 
